# -*- coding: utf-8 -*-

import bz2
import csv

class OutputWriter(object):

    def __init__(self, metapopulation, filename, fieldnames, header=True,
                 include_uuid=False, compress=False):
        self.metapopulation = metapopulation
        self.filename = filename
        self.fieldnames = fieldnames
        self.include_uuid = include_uuid
        self.compress = compress

        if compress:
            self.filename += ".bz2"

            try:
                self.outfile = bz2.open(filename=self.filename, mode='wt',
                                        newline='', encoding='UTF-8')
            except AttributeError:
                self.outfile = bz2.BZ2File(self.filename, 'w')

        else:
            self.outfile = open(self.filename, mode='wt')

        if self.include_uuid:
            self.fieldnames.insert(0, 'UUID')

        self.writer = csv.DictWriter(self.outfile, fieldnames=self.fieldnames)

        if header:
            self.writer.writeheader()

    def update(self, time):
        pass

    def writerow(self, record):
        if self.include_uuid:
            record['UUID'] = self.metapopulation.config['Simulation']['UUID']

        self.writer.writerow(record)

    def close(self):
        self.outfile.close()




# -*- coding: utf-8 -*-

from hankshaw.OutputWriter import OutputWriter


class EnvChangeOutput(OutputWriter):

    def __init__(self, metapopulation, filename='environmental_change.csv',
                 header=True, include_uuid=False, compress=False):
        fieldnames = ['Time', 'EnvironmentChanged']
        super(EnvChangeOutput, self).__init__(metapopulation=metapopulation,
                                              filename=filename,
                                              fieldnames=fieldnames,
                                              header=header,
                                              include_uuid=include_uuid,
                                              compress=compress)

    def update(self, time):
        record = {'Time': time,
                  'EnvironmentChanged': self.metapopulation.environment_changed}
        self.writerow(record)



# -*- coding: utf-8 -*-

from hankshaw.OutputWriter import OutputWriter


class MetapopulationOutput(OutputWriter):

    def __init__(self, metapopulation, filename='metapopulation.csv',
                 header=True, include_uuid=False, compress=False):
        fieldnames = ['Time', 'Births', 'Size', 'CooperatorProportion', 'MaxCooperatorFitness', 'MaxDefectorFitness']
        super(MetapopulationOutput, self).__init__(metapopulation=metapopulation,
                                                   filename=filename,
                                                   fieldnames=fieldnames,
                                                   header=header,
                                                   include_uuid=include_uuid,
                                                   compress=compress)


    def update(self, time):
        fits = self.metapopulation.max_fitnesses()
        record = {'Time': time,
                  'Births': self.metapopulation.num_births,
                  'Size': self.metapopulation.size(),
                  'CooperatorProportion': self.metapopulation.prop_producers(),
                  'MaxCooperatorFitness': max(fits[0]),
                  'MaxDefectorFitness': max(fits[1])}
        self.writerow(record)




# -*- coding: utf-8 -*-

from hankshaw.OutputWriter import OutputWriter


class FitnessOutput(OutputWriter):

    def __init__(self, metapopulation, filename='max_fitness.csv',
                 header=True, include_uuid=False, compress=False):
        fieldnames = ['Time', 'Producers', 'Nonproducers']
        super(FitnessOutput, self).__init__(metapopulation=metapopulation,
                                            filename=filename,
                                            fieldnames=fieldnames,
                                            header=header,
                                            include_uuid=include_uuid,
                                            compress=compress)

    def update(self, time):
        maxfit = self.metapopulation.max_fitnesses()
        record = {'Time': time,
                  'Producers': max(maxfit[0]),
                  'Nonproducers': max(maxfit[1])}
        self.writerow(record)




# -*- coding: utf-8 -*-

from hankshaw.OutputWriter import OutputWriter


class PopulationOutput(OutputWriter):

    def __init__(self, metapopulation, filename='population.csv',
                 header=True, include_uuid=False, compress=False):
        fieldnames = ['Time', 'Population', 'Size', 'Producers',
                      'PropProducers', 'NonProducers', 'PropNonProducers',
                      'AvgFitness']
        super(PopulationOutput, self).__init__(metapopulation=metapopulation,
                                               filename=filename,
                                               fieldnames=fieldnames,
                                               header=header,
                                               include_uuid=include_uuid,
                                               compress=compress)

    def update(self, time):
        for n, d in self.metapopulation.topology.nodes_iter(data=True):
            size = len(d['population'])

            if size == 0:
                num_producers = 0
                num_nonproducers = 0
                prop_producers = 'NA'
                prop_nonproducers = 'NA'
                average_fitness = 'NA'
            else:
                num_producers = d['population'].num_producers()
                num_nonproducers = size - num_producers
                prop_producers = 1.0*num_producers/size
                prop_nonproducers = 1.0*num_nonproducers/size
                average_fitness = d['population'].average_fitness()

            record = {'Time': time,
                      'Population': n,
                      'Size': size, 
                      'Producers': num_producers,
                      'PropProducers': prop_producers,
                      'NonProducers': num_nonproducers,
                      'PropNonProducers': prop_nonproducers,
                      'AvgFitness': average_fitness}
            self.writerow(record)

